@@ -19,18 +19,18 @@ package org.gradle.composite.internal;
 import com.google.common.collect.Sets;
 import org.gradle.StartParameter;
 import org.gradle.api.InvalidUserDataException;
-import org.gradle.api.initialization.ConfigurableIncludedBuild;
-import org.gradle.api.initialization.IncludedBuild;
 import org.gradle.api.initialization.Settings;
 import org.gradle.api.internal.SettingsInternal;
 import org.gradle.api.internal.artifacts.ImmutableModuleIdentifierFactory;
+import org.gradle.api.initialization.ConfigurableIncludedBuild;
+import org.gradle.api.initialization.IncludedBuild;
 import org.gradle.initialization.GradleLauncher;
-import org.gradle.initialization.IncludedBuildFactory;
 import org.gradle.initialization.NestedBuildFactory;
 import org.gradle.internal.Factory;
 import org.gradle.internal.concurrent.CompositeStoppable;
 import org.gradle.internal.concurrent.Stoppable;
 import org.gradle.internal.reflect.Instantiator;
+import org.gradle.internal.work.WorkerLeaseService;
 
 import java.io.File;
 import java.util.Set;
@@ -41,12 +41,14 @@ public class DefaultIncludedBuildFactory implements IncludedBuildFactory, Stoppa
     private final NestedBuildFactory nestedBuildFactory;
     private final Set<GradleLauncher> launchers = Sets.newHashSet();
     private final ImmutableModuleIdentifierFactory moduleIdentifierFactory;
+    private final WorkerLeaseService workerLeaseService;
 
-    public DefaultIncludedBuildFactory(Instantiator instantiator, StartParameter startParameter, NestedBuildFactory nestedBuildFactory, ImmutableModuleIdentifierFactory moduleIdentifierFactory) {
+    public DefaultIncludedBuildFactory(Instantiator instantiator, StartParameter startParameter, NestedBuildFactory nestedBuildFactory, ImmutableModuleIdentifierFactory moduleIdentifierFactory, WorkerLeaseService workerLeaseService) {
         this.instantiator = instantiator;
         this.startParameter = startParameter;
         this.nestedBuildFactory = nestedBuildFactory;
         this.moduleIdentifierFactory = moduleIdentifierFactory;
+        this.workerLeaseService = workerLeaseService;
     }
 
     private void validateBuildDirectory(File dir) {
@@ -71,7 +73,7 @@ public class DefaultIncludedBuildFactory implements IncludedBuildFactory, Stoppa
     public ConfigurableIncludedBuild createBuild(File buildDirectory) {
         validateBuildDirectory(buildDirectory);
         Factory<GradleLauncher> factory = new ContextualGradleLauncherFactory(buildDirectory, nestedBuildFactory, startParameter);
-        DefaultIncludedBuild includedBuild = instantiator.newInstance(DefaultIncludedBuild.class, buildDirectory, factory, moduleIdentifierFactory);
+        DefaultIncludedBuild includedBuild = instantiator.newInstance(DefaultIncludedBuild.class, buildDirectory, factory, moduleIdentifierFactory, workerLeaseService.getCurrentWorkerLease());
 
         SettingsInternal settingsInternal = includedBuild.getLoadedSettings();
         validateIncludedBuild(includedBuild, settingsInternal);
