@@ -19,6 +19,7 @@ import groovy.lang.Closure;
 import groovy.lang.DelegatesTo;
 import org.gradle.api.Action;
 import org.gradle.api.Incubating;
+import org.gradle.api.attributes.HasConfigurableAttributes;
 import org.gradle.api.file.FileCollection;
 import org.gradle.api.specs.Spec;
 import org.gradle.api.tasks.TaskDependency;
@@ -40,10 +41,9 @@ import static groovy.lang.Closure.DELEGATE_FIRST;
  * If you want to refer to the artifacts declared in this configuration
  * please use {@link #getArtifacts()} or {@link #getAllArtifacts()}.
  * Read more about declaring artifacts in the configuration in docs for {@link org.gradle.api.artifacts.dsl.ArtifactHandler}
- * <p>
  */
 @HasInternalProtocol
-public interface Configuration extends FileCollection {
+public interface Configuration extends FileCollection, HasConfigurableAttributes<Configuration> {
 
     /**
      * Returns the resolution strategy used by this configuration.
@@ -195,7 +195,7 @@ public interface Configuration extends FileCollection {
     Set<File> resolve();
 
     /**
-     * Takes a closure which gets coerced into a Spec. Behaves otherwise in the same way as
+     * Takes a closure which gets coerced into a {@link Spec}. Behaves otherwise in the same way as
      * {@link #files(org.gradle.api.specs.Spec)}.
      *
      * @param dependencySpecClosure The closure describing a filter applied to the all the dependencies of this configuration (including dependencies from extended configurations).
@@ -218,13 +218,13 @@ public interface Configuration extends FileCollection {
      * But only the resulting set of files belonging to the specified dependencies
      * is returned.
      *
-     * @param dependencies The dependences to be resolved
+     * @param dependencies The dependencies to be resolved
      * @return The files of a subset of dependencies of this configuration.
      */
     Set<File> files(Dependency... dependencies);
 
     /**
-     * Resolves this configuration lazily. The resolve happens when the elements of the returned FileCollection get accessed the first time.
+     * Resolves this configuration lazily. The resolve happens when the elements of the returned {@link FileCollection} get accessed the first time.
      * This locates and downloads the files which make up this configuration. Only the resulting set of files belonging to the subset
      * of dependencies specified by the dependencySpec is contained in the FileCollection.
      *
@@ -234,7 +234,7 @@ public interface Configuration extends FileCollection {
     FileCollection fileCollection(Spec<? super Dependency> dependencySpec);
 
     /**
-     * Takes a closure which gets coerced into a Spec. Behaves otherwise in the same way as
+     * Takes a closure which gets coerced into a {@link Spec}. Behaves otherwise in the same way as
      * {@link #fileCollection(org.gradle.api.specs.Spec)}.
      *
      * @param dependencySpecClosure The closure describing a filter applied to the all the dependencies of this configuration (including dependencies from extended configurations).
@@ -243,7 +243,7 @@ public interface Configuration extends FileCollection {
     FileCollection fileCollection(Closure dependencySpecClosure);
 
     /**
-     * Resolves this configuration lazily. The resolve happens when the elements of the returned FileCollection get accessed the first time.
+     * Resolves this configuration lazily. The resolve happens when the elements of the returned {@link FileCollection} get accessed the first time.
      * This locates and downloads the files which make up this configuration. Only the resulting set of files belonging to specified
      * dependencies is contained in the FileCollection.
      *
@@ -254,7 +254,7 @@ public interface Configuration extends FileCollection {
 
     /**
      * Resolves this configuration. This locates and downloads the files which make up this configuration, and returns
-     * a ResolvedConfiguration that may be used to determine information about the resolve (including errors).
+     * a {@link ResolvedConfiguration} that may be used to determine information about the resolve (including errors).
      *
      * @return The ResolvedConfiguration object
      */
@@ -350,7 +350,7 @@ public interface Configuration extends FileCollection {
      * This method is useful for specifying default dependencies for a configuration:
      * <pre autoTested='true'>
      * configurations { conf }
-     * configurations['conf'].defaultDependencies { dependencies ->
+     * configurations['conf'].defaultDependencies { dependencies -&gt;
      *      dependencies.add(owner.project.dependencies.create("org.gradle:my-util:1.0"))
      * }
      * </pre>
@@ -381,6 +381,24 @@ public interface Configuration extends FileCollection {
      */
     ResolvableDependencies getIncoming();
 
+    /**
+     * Returns the outgoing artifacts of this configuration.
+     *
+     * @return The outgoing artifacts of this configuration.
+     * @since 3.4
+     */
+    @Incubating
+    ConfigurationPublications getOutgoing();
+
+    /**
+     * Configures the outgoing artifacts of this configuration.
+     *
+     * @param action The action to perform the configuration.
+     * @since 3.4
+     */
+    @Incubating
+    void outgoing(Action<? super ConfigurationPublications> action);
+
     /**
      * Creates a copy of this configuration that only contains the dependencies directly in this configuration
      * (without contributions from superconfigurations).  The new configuration will be in the
@@ -418,7 +436,7 @@ public interface Configuration extends FileCollection {
     Configuration copyRecursive(Spec<? super Dependency> dependencySpec);
 
     /**
-     * Takes a closure which gets coerced into a Spec. Behaves otherwise in the same way as {@link #copy(org.gradle.api.specs.Spec)}
+     * Takes a closure which gets coerced into a {@link Spec}. Behaves otherwise in the same way as {@link #copy(org.gradle.api.specs.Spec)}
      *
      * @param dependencySpec filtering requirements
      * @return copy of this configuration
@@ -426,10 +444,43 @@ public interface Configuration extends FileCollection {
     Configuration copy(Closure dependencySpec);
 
     /**
-     * Takes a closure which gets coerced into a Spec. Behaves otherwise in the same way as {@link #copyRecursive(org.gradle.api.specs.Spec)}
+     * Takes a closure which gets coerced into a {@link Spec}. Behaves otherwise in the same way as {@link #copyRecursive(org.gradle.api.specs.Spec)}
      *
      * @param dependencySpec filtering requirements
      * @return copy of this configuration
      */
     Configuration copyRecursive(Closure dependencySpec);
+
+    /**
+     * Configures if a configuration can be consumed.
+     *
+     * @since 3.3
+     */
+    @Incubating
+    void setCanBeConsumed(boolean allowed);
+
+    /**
+     * Returns true if this configuration can be consumed from another project, or published. Defaults to true.
+     * @return true if this configuration can be consumed or published.
+     * @since 3.3
+     */
+    @Incubating
+    boolean isCanBeConsumed();
+
+    /**
+     * Configures if a configuration can be resolved.
+     *
+     * @since 3.3
+     */
+    @Incubating
+    void setCanBeResolved(boolean allowed);
+
+    /**
+     * Returns true if it is allowed to query or resolve this configuration. Defaults to true.
+     * @return true if this configuration can be queried or resolved.
+     * @since 3.3
+     */
+    @Incubating
+    boolean isCanBeResolved();
+
 }
