@@ -17,9 +17,17 @@ package org.gradle.api.internal;
 
 import org.gradle.api.file.FileCollection;
 
+import javax.annotation.Nullable;
+
 public interface TaskExecutionHistory {
     /**
      * Returns the set of output files which the task produced.
      */
     FileCollection getOutputFiles();
+
+    /**
+     * Returns if overlapping outputs were detected
+     */
+    @Nullable
+    OverlappingOutputs getOverlappingOutputs();
 }
