@@ -15,12 +15,12 @@
  */
 package org.gradle.api.internal.changedetection;
 
-import org.gradle.api.Nullable;
 import org.gradle.api.internal.TaskExecutionHistory;
 import org.gradle.api.tasks.incremental.IncrementalTaskInputs;
 import org.gradle.caching.internal.tasks.TaskOutputCachingBuildCacheKey;
 import org.gradle.internal.id.UniqueId;
 
+import javax.annotation.Nullable;
 import java.util.Collection;
 
 /**
@@ -47,19 +47,19 @@ public interface TaskArtifactState {
     TaskOutputCachingBuildCacheKey calculateCacheKey();
 
     /**
-     * Called before the task is to be executed. Note that {@link #isUpToDate(java.util.Collection)} may not necessarily have been called.
+     * Ensure snapshot is taken of the task's inputs and outputs before it is executed.
      */
-    void beforeTask();
+    void ensureSnapshotBeforeTask();
 
     /**
-     * Called on successful completion of task execution.
+     * Retakes output file snapshots and prevents the task from executing in an incremental fashion.
      */
-    void afterTask();
+    void afterOutputsRemovedBeforeTask();
 
     /**
-     * Called when this state is finished with.
+     * Called on completion of task execution.
      */
-    void finished();
+    void afterTask(Throwable failure);
 
     /**
      * Returns the history for this task.
