@@ -16,34 +16,28 @@
 package org.gradle.api.internal.changedetection.state;
 
 import org.gradle.api.file.FileCollection;
-import org.gradle.api.internal.tasks.TaskFilePropertySpec;
 import org.gradle.internal.serialize.SerializerRegistry;
+import org.gradle.normalization.internal.InputNormalizationStrategy;
 
 public interface FileCollectionSnapshotter {
+    /**
+     * The type used to refer to this snapshotter in the {@link FileCollectionSnapshotterRegistry}.
+     * Must be a super-type of the actual implementation.
+     */
+    Class<? extends FileCollectionSnapshotter> getRegisteredType();
+
     /**
      * Registers the serializer(s) that can be used to serialize the {@link FileCollectionSnapshot} implementations produced by this snapshotter.
      */
     void registerSerializers(SerializerRegistry registry);
 
-    /**
-     * Creates an empty snapshot, which changes can be later merged into.
-     *
-     * @return The snapshot.
-     */
-    FileCollectionSnapshot emptySnapshot();
-
     /**
      * Creates a snapshot of the contents of the given collection.
      *
      * @param files The files to snapshot.
-     * @param compareStrategy How to compare this collection snapshot to others.
-     * @param pathSensitivity How to compare file paths.
+     * @param snapshotNormalizationStrategy How to normalize file snapshots.
+     * @param normalizationStrategy The input normalization strategy to take into account.
      * @return The snapshot.
      */
-    FileCollectionSnapshot snapshot(FileCollection files, TaskFilePropertyCompareStrategy compareStrategy, TaskFilePropertyPathSensitivity pathSensitivity);
-
-    /**
-     * Creates a snapshot of the contents of the given property.
-     */
-    FileCollectionSnapshot snapshot(TaskFilePropertySpec propertySpec);
+    FileCollectionSnapshot snapshot(FileCollection files, SnapshotNormalizationStrategy snapshotNormalizationStrategy, InputNormalizationStrategy normalizationStrategy);
 }
