@@ -15,9 +15,10 @@
  */
 package org.gradle.api.internal.changedetection.state;
 
-import org.gradle.api.Nullable;
 import org.gradle.api.internal.TaskInternal;
 
+import javax.annotation.Nullable;
+
 public interface TaskHistoryRepository {
     History getHistory(TaskInternal task);
 
