@@ -15,11 +15,12 @@
  */
 package org.gradle.cache;
 
-import org.gradle.api.Nullable;
 import org.gradle.cache.internal.CacheDecorator;
 import org.gradle.internal.serialize.BaseSerializerFactory;
 import org.gradle.internal.serialize.Serializer;
 
+import javax.annotation.Nullable;
+
 public class PersistentIndexedCacheParameters<K, V> {
     private static final BaseSerializerFactory SERIALIZER_FACTORY = new BaseSerializerFactory();
     private final String cacheName;
