@@ -29,6 +29,11 @@ public class DefaultMultiProcessSafePersistentIndexedCache<K, V> implements Mult
         this.fileAccess = fileAccess;
     }
 
+    @Override
+    public String toString() {
+        return fileAccess.toString();
+    }
+
     @Override
     public V get(final K key) {
         final BTreePersistentIndexedCache<K, V> cache = getCache();
