@@ -20,8 +20,8 @@ import com.google.common.collect.ImmutableList;
 import com.google.common.collect.ImmutableSortedMap;
 import com.google.common.collect.ImmutableSortedSet;
 import com.google.common.hash.HashCode;
-import org.gradle.api.Nullable;
 
+import javax.annotation.Nullable;
 import java.util.List;
 
 /**
