@@ -17,9 +17,11 @@
 package org.gradle.caching.local;
 
 import org.gradle.api.Incubating;
-import org.gradle.api.Nullable;
 import org.gradle.caching.configuration.AbstractBuildCache;
 
+import javax.annotation.Nullable;
+
+
 /**
  * Configuration object for the local directory build cache.
  *
@@ -28,6 +30,7 @@ import org.gradle.caching.configuration.AbstractBuildCache;
 @Incubating
 public class DirectoryBuildCache extends AbstractBuildCache {
     private Object directory;
+    private long targetSizeInMB = 5 * 1024; // 5 GB
 
     /**
      * Returns the directory to use to store the build cache.
@@ -45,4 +48,31 @@ public class DirectoryBuildCache extends AbstractBuildCache {
     public void setDirectory(Object directory) {
         this.directory = directory;
     }
+
+    /**
+     * The target size of the build cache in megabytes.
+     * Defaults to 5 GB.
+     * <p>
+     * Must be greater than or equal to 1, although larger cache sizes will be more useful.
+     *
+     * @since 4.0
+     */
+    public long getTargetSizeInMB() {
+        return targetSizeInMB;
+    }
+
+    /**
+     * The target size of the build cache in megabytes.
+     * Defaults to 5 GB.
+     * <p>
+     * Must be greater than or equal to 1, although larger cache sizes will be more useful.
+     *
+     * @since 4.0
+     */
+    public void setTargetSizeInMB(long targetSizeInMB) {
+        if (targetSizeInMB < 1) {
+            throw new IllegalArgumentException("Directory build cache needs to have at least 1 MB of space but more space is useful.");
+        }
+        this.targetSizeInMB = targetSizeInMB;
+    }
 }
