@@ -37,9 +37,14 @@ import org.gradle.internal.concurrent.Stoppable;
 import org.gradle.internal.event.ListenerManager;
 import org.gradle.internal.featurelifecycle.LoggingDeprecatedFeatureHandler;
 import org.gradle.internal.featurelifecycle.ScriptUsageLocationReporter;
+import org.gradle.internal.invocation.BuildController;
+import org.gradle.internal.invocation.GradleBuildController;
 import org.gradle.internal.logging.progress.ProgressLoggerFactory;
 import org.gradle.internal.logging.text.StyledTextOutputFactory;
+import org.gradle.internal.operations.BuildOperationContext;
 import org.gradle.internal.operations.BuildOperationExecutor;
+import org.gradle.internal.operations.CallableBuildOperation;
+import org.gradle.internal.progress.BuildOperationDescriptor;
 import org.gradle.internal.progress.BuildProgressFilter;
 import org.gradle.internal.progress.BuildProgressLogger;
 import org.gradle.internal.progress.LoggerProvider;
@@ -47,6 +52,7 @@ import org.gradle.internal.reflect.Instantiator;
 import org.gradle.internal.service.ServiceRegistry;
 import org.gradle.internal.service.scopes.BuildScopeServices;
 import org.gradle.internal.service.scopes.BuildSessionScopeServices;
+import org.gradle.internal.service.scopes.BuildTreeScopeServices;
 import org.gradle.internal.service.scopes.GradleUserHomeScopeServiceRegistry;
 import org.gradle.internal.service.scopes.ServiceRegistryFactory;
 import org.gradle.invocation.DefaultGradle;
@@ -54,6 +60,7 @@ import org.gradle.profile.ProfileEventAdapter;
 import org.gradle.profile.ReportGeneratingProfileListener;
 import org.gradle.util.DeprecationLogger;
 
+import javax.annotation.Nullable;
 import java.util.List;
 
 public class DefaultGradleLauncherFactory implements GradleLauncherFactory {
@@ -71,13 +78,12 @@ public class DefaultGradleLauncherFactory implements GradleLauncherFactory {
         listenerManager.useLogger(new ProjectEvaluationLogger(progressLoggerFactory));
     }
 
-    private GradleLauncher createChildInstance(StartParameter startParameter, GradleLauncher parent, BuildSessionScopeServices buildSessionScopeServices, List<?> servicesToStop) {
+    private GradleLauncher createChildInstance(StartParameter startParameter, GradleLauncher parent, BuildTreeScopeServices buildTreeScopeServices, List<?> servicesToStop) {
         ServiceRegistry services = parent.getGradle().getServices();
         BuildRequestMetaData requestMetaData = new DefaultBuildRequestMetaData(services.get(BuildClientMetaData.class));
         BuildCancellationToken cancellationToken = services.get(BuildCancellationToken.class);
         BuildEventConsumer buildEventConsumer = services.get(BuildEventConsumer.class);
-
-        return doNewInstance(startParameter, parent, cancellationToken, requestMetaData, buildEventConsumer, buildSessionScopeServices, servicesToStop);
+        return doNewInstance(startParameter, parent, cancellationToken, requestMetaData, buildEventConsumer, buildTreeScopeServices, servicesToStop);
     }
 
     @Override
@@ -87,12 +93,12 @@ public class DefaultGradleLauncherFactory implements GradleLauncherFactory {
             throw new IllegalStateException("Cannot have a current build");
         }
 
-        if (!(parentRegistry instanceof BuildSessionScopeServices)) {
-            throw new IllegalArgumentException("Service registry must be of build session scope");
+        if (!(parentRegistry instanceof BuildTreeScopeServices)) {
+            throw new IllegalArgumentException("Service registry must be of build-tree scope");
         }
-        BuildSessionScopeServices sessionScopeServices = (BuildSessionScopeServices) parentRegistry;
+        BuildTreeScopeServices buildTreeScopeServices = (BuildTreeScopeServices) parentRegistry;
 
-        DefaultGradleLauncher launcher = doNewInstance(startParameter, null, requestContext.getCancellationToken(), requestContext, requestContext.getEventConsumer(), sessionScopeServices, ImmutableList.of(new Stoppable() {
+        DefaultGradleLauncher launcher = doNewInstance(startParameter, null, requestContext.getCancellationToken(), requestContext, requestContext.getEventConsumer(), buildTreeScopeServices, ImmutableList.of(new Stoppable() {
             @Override
             public void stop() {
                 rootBuild = null;
@@ -107,13 +113,14 @@ public class DefaultGradleLauncherFactory implements GradleLauncherFactory {
     }
 
     private DefaultGradleLauncher doNewInstance(StartParameter startParameter, GradleLauncher parent,
-                                                BuildCancellationToken cancellationToken, BuildRequestMetaData requestMetaData, BuildEventConsumer buildEventConsumer, final BuildSessionScopeServices sessionScopeServices, List<?> servicesToStop) {
-        BuildScopeServices serviceRegistry = BuildScopeServices.forSession(sessionScopeServices);
+                                                BuildCancellationToken cancellationToken, BuildRequestMetaData requestMetaData, BuildEventConsumer buildEventConsumer,
+                                                final BuildTreeScopeServices buildTreeScopeServices, List<?> servicesToStop) {
+        BuildScopeServices serviceRegistry = new BuildScopeServices(buildTreeScopeServices);
         serviceRegistry.add(BuildRequestMetaData.class, requestMetaData);
         serviceRegistry.add(BuildClientMetaData.class, requestMetaData.getClient());
         serviceRegistry.add(BuildEventConsumer.class, buildEventConsumer);
         serviceRegistry.add(BuildCancellationToken.class, cancellationToken);
-        NestedBuildFactoryImpl nestedBuildFactory = new NestedBuildFactoryImpl(sessionScopeServices);
+        NestedBuildFactoryImpl nestedBuildFactory = new NestedBuildFactoryImpl(buildTreeScopeServices);
         serviceRegistry.add(NestedBuildFactory.class, nestedBuildFactory);
 
         ListenerManager listenerManager = serviceRegistry.get(ListenerManager.class);
@@ -172,32 +179,89 @@ public class DefaultGradleLauncherFactory implements GradleLauncherFactory {
     }
 
     private class NestedBuildFactoryImpl implements NestedBuildFactory {
-        private final BuildSessionScopeServices sessionScopeServices;
+        private final BuildTreeScopeServices buildTreeScopeServices;
         private DefaultGradleLauncher parent;
 
-        public NestedBuildFactoryImpl(BuildSessionScopeServices sessionScopeServices) {
-            this.sessionScopeServices = sessionScopeServices;
+        public NestedBuildFactoryImpl(BuildTreeScopeServices buildTreeScopeServices) {
+            this.buildTreeScopeServices = buildTreeScopeServices;
         }
 
         @Override
         public GradleLauncher nestedInstance(StartParameter startParameter) {
-            return createChildInstance(startParameter, parent, sessionScopeServices, ImmutableList.of());
+            return createChildInstance(startParameter, parent, buildTreeScopeServices, ImmutableList.of());
         }
 
         @Override
-        public GradleLauncher nestedInstanceWithNewSession(StartParameter startParameter) {
+        public BuildController nestedBuildController(StartParameter startParameter) {
             final ServiceRegistry userHomeServices = userHomeDirServiceRegistry.getServicesFor(startParameter.getGradleUserHomeDir());
             BuildSessionScopeServices sessionScopeServices = new BuildSessionScopeServices(userHomeServices, startParameter, ClassPath.EMPTY);
-            return createChildInstance(startParameter, parent, sessionScopeServices, ImmutableList.of(sessionScopeServices, new Stoppable() {
+            BuildTreeScopeServices buildTreeScopeServices = new BuildTreeScopeServices(sessionScopeServices);
+            GradleLauncher childInstance = createChildInstance(startParameter, parent, buildTreeScopeServices, ImmutableList.of(buildTreeScopeServices, sessionScopeServices, new Stoppable() {
                 @Override
                 public void stop() {
                     userHomeDirServiceRegistry.release(userHomeServices);
                 }
             }));
+            return new NestedBuildController(new GradleBuildController(childInstance));
         }
 
         public void setParent(DefaultGradleLauncher parent) {
             this.parent = parent;
         }
+
+        private class NestedBuildController implements BuildController {
+            private final BuildController delegate;
+
+            NestedBuildController(BuildController delegate) {
+                this.delegate = delegate;
+            }
+
+            @Override
+            public void stop() {
+                delegate.stop();
+            }
+
+            @Override
+            public GradleInternal getGradle() {
+                return delegate.getGradle();
+            }
+
+            @Override
+            public GradleInternal run() {
+                BuildOperationExecutor executor = getGradle().getServices().get(BuildOperationExecutor.class);
+                return executor.call(new CallableBuildOperation<GradleInternal>() {
+                    @Override
+                    public GradleInternal call(BuildOperationContext context) {
+                        return delegate.run();
+                    }
+
+                    @Override
+                    public BuildOperationDescriptor.Builder description() {
+                        return BuildOperationDescriptor.displayName("Run nested build").parent(parent.getGradle().getBuildOperation());
+                    }
+                });
+            }
+
+            @Override
+            public GradleInternal configure() {
+                throw new UnsupportedOperationException();
+            }
+
+            @Override
+            public boolean hasResult() {
+                return delegate.hasResult();
+            }
+
+            @Nullable
+            @Override
+            public Object getResult() {
+                return delegate.getResult();
+            }
+
+            @Override
+            public void setResult(@Nullable Object result) {
+                delegate.setResult(result);
+            }
+        }
     }
 }
