@@ -17,7 +17,6 @@
 package org.gradle.initialization.buildsrc;
 
 import org.gradle.StartParameter;
-import org.gradle.api.Transformer;
 import org.gradle.api.internal.GradleInternal;
 import org.gradle.api.internal.initialization.ClassLoaderScope;
 import org.gradle.cache.CacheBuilder;
@@ -28,10 +27,12 @@ import org.gradle.initialization.GradleLauncher;
 import org.gradle.initialization.NestedBuildFactory;
 import org.gradle.internal.classpath.CachedClasspathTransformer;
 import org.gradle.internal.classpath.ClassPath;
-import org.gradle.internal.classpath.DefaultClassPath;
+import org.gradle.internal.invocation.BuildController;
+import org.gradle.internal.invocation.GradleBuildController;
 import org.gradle.internal.operations.BuildOperationContext;
-import org.gradle.internal.progress.BuildOperationDetails;
-import org.gradle.internal.progress.BuildOperationExecutor;
+import org.gradle.internal.operations.BuildOperationExecutor;
+import org.gradle.internal.operations.CallableBuildOperation;
+import org.gradle.internal.progress.BuildOperationDescriptor;
 import org.gradle.util.GradleVersion;
 import org.gradle.util.Path;
 import org.slf4j.Logger;
@@ -50,21 +51,22 @@ public class BuildSourceBuilder {
     private final CacheRepository cacheRepository;
     private final BuildOperationExecutor buildOperationExecutor;
     private final CachedClasspathTransformer cachedClasspathTransformer;
+    private final BuildSrcBuildListenerFactory buildSrcBuildListenerFactory;
 
-    public BuildSourceBuilder(NestedBuildFactory nestedBuildFactory, ClassLoaderScope classLoaderScope, CacheRepository cacheRepository, BuildOperationExecutor buildOperationExecutor, CachedClasspathTransformer cachedClasspathTransformer) {
+    public BuildSourceBuilder(NestedBuildFactory nestedBuildFactory, ClassLoaderScope classLoaderScope, CacheRepository cacheRepository, BuildOperationExecutor buildOperationExecutor, CachedClasspathTransformer cachedClasspathTransformer, BuildSrcBuildListenerFactory buildSrcBuildListenerFactory) {
         this.nestedBuildFactory = nestedBuildFactory;
         this.classLoaderScope = classLoaderScope;
         this.cacheRepository = cacheRepository;
         this.buildOperationExecutor = buildOperationExecutor;
         this.cachedClasspathTransformer = cachedClasspathTransformer;
+        this.buildSrcBuildListenerFactory = buildSrcBuildListenerFactory;
     }
 
     public ClassLoaderScope buildAndCreateClassLoader(StartParameter startParameter) {
         ClassPath classpath = createBuildSourceClasspath(startParameter);
-        ClassLoaderScope childScope = classLoaderScope.createChild(startParameter.getCurrentDir().getAbsolutePath());
-        childScope.export(cachedClasspathTransformer.transform(classpath));
-        childScope.lock();
-        return childScope;
+        return classLoaderScope.createChild(startParameter.getCurrentDir().getAbsolutePath())
+            .export(cachedClasspathTransformer.transform(classpath))
+            .lock();
     }
 
     ClassPath createBuildSourceClasspath(final StartParameter startParameter) {
@@ -73,13 +75,18 @@ public class BuildSourceBuilder {
         LOGGER.debug("Starting to build the build sources.");
         if (!startParameter.getCurrentDir().isDirectory()) {
             LOGGER.debug("Gradle source dir does not exist. We leave.");
-            return new DefaultClassPath();
+            return ClassPath.EMPTY;
         }
-        return buildOperationExecutor.run(BuildOperationDetails.displayName("Build buildSrc").progressDisplayName("buildSrc").build(), new Transformer<ClassPath, BuildOperationContext>() {
+        return buildOperationExecutor.call(new CallableBuildOperation<ClassPath>() {
             @Override
-            public ClassPath transform(BuildOperationContext buildOperationContext) {
+            public ClassPath call(BuildOperationContext context) {
                 return buildBuildSrc(startParameter);
             }
+
+            @Override
+            public BuildOperationDescriptor.Builder description() {
+                return BuildOperationDescriptor.displayName("Build buildSrc").progressDisplayName("buildSrc");
+            }
         });
     }
 
@@ -88,11 +95,11 @@ public class BuildSourceBuilder {
         // Otherwise, just to a regular build
         final PersistentCache buildSrcCache = createCache(startParameter);
         try {
-            GradleLauncher gradleLauncher = buildGradleLauncher(startParameter);
+            BuildController buildController = createBuildController(startParameter);
             try {
-                return buildSrcCache.useCache("rebuild buildSrc", new BuildSrcUpdateFactory(buildSrcCache, gradleLauncher, new BuildSrcBuildListenerFactory()));
+                return buildSrcCache.useCache(new BuildSrcUpdateFactory(buildSrcCache, buildController, buildSrcBuildListenerFactory));
             } finally {
-                gradleLauncher.stop();
+                buildController.stop();
             }
         } finally {
             // This isn't quite right. We should not unlock the classes until we're finished with them, and the classes may be used across multiple builds
@@ -102,12 +109,17 @@ public class BuildSourceBuilder {
 
     PersistentCache createCache(StartParameter startParameter) {
         return cacheRepository
-                .cache(new File(startParameter.getCurrentDir(), ".gradle/noVersion/buildSrc"))
-                .withCrossVersionCache(CacheBuilder.LockTarget.CachePropertiesFile)
-                .withDisplayName("buildSrc state cache")
-                .withLockOptions(mode(FileLockManager.LockMode.None).useCrossVersionImplementation())
-                .withProperties(Collections.singletonMap("gradle.version", GradleVersion.current().getVersion()))
-                .open();
+            .cache(new File(startParameter.getCurrentDir(), ".gradle/noVersion/buildSrc"))
+            .withCrossVersionCache(CacheBuilder.LockTarget.CachePropertiesFile)
+            .withDisplayName("buildSrc state cache")
+            .withLockOptions(mode(FileLockManager.LockMode.None).useCrossVersionImplementation())
+            .withProperties(Collections.singletonMap("gradle.version", GradleVersion.current().getVersion()))
+            .open();
+    }
+
+    private BuildController createBuildController(StartParameter startParameter) {
+        GradleLauncher gradleLauncher = buildGradleLauncher(startParameter);
+        return new GradleBuildController(gradleLauncher);
     }
 
     private GradleLauncher buildGradleLauncher(StartParameter startParameter) {
