@@ -16,8 +16,12 @@
 
 package org.gradle.api.internal.artifacts.ivyservice.projectmodule;
 
-import org.gradle.internal.component.model.ComponentArtifactMetaData;
+import org.gradle.internal.component.model.ComponentArtifactMetadata;
 
 public interface ProjectArtifactBuilder {
-    void build(ComponentArtifactMetaData artifact);
+
+    /**
+     * @param artifact Actually build an artifact that is included in dependency resolution.
+     */
+    void build(ComponentArtifactMetadata artifact);
 }
