@@ -16,53 +16,61 @@
 
 package org.gradle.api.internal.artifacts.repositories.resolver;
 
+import com.google.common.base.Charsets;
+import com.google.common.hash.Hashing;
 import org.apache.ivy.util.ContextualSAXHandler;
 import org.apache.ivy.util.XMLHelper;
 import org.gradle.api.resources.MissingResourceException;
 import org.gradle.api.resources.ResourceException;
 import org.gradle.internal.ErroringAction;
 import org.gradle.internal.resource.ExternalResource;
-import org.gradle.internal.resource.transport.ExternalResourceRepository;
+import org.gradle.internal.resource.ExternalResourceName;
+import org.gradle.internal.resource.local.FileStore;
+import org.gradle.internal.resource.local.LocallyAvailableResource;
+import org.gradle.internal.resource.transfer.CacheAwareExternalResourceAccessor;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 import org.xml.sax.SAXException;
 
 import javax.xml.parsers.ParserConfigurationException;
+import java.io.File;
 import java.io.IOException;
 import java.io.InputStream;
-import java.net.URI;
 
 class MavenMetadataLoader {
     private static final Logger LOGGER = LoggerFactory.getLogger(MavenMetadataLoader.class);
+    private final CacheAwareExternalResourceAccessor cacheAwareExternalResourceAccessor;
+    private final FileStore<String> resourcesFileStore;
 
-    private final ExternalResourceRepository repository;
-
-    public MavenMetadataLoader(ExternalResourceRepository repository) {
-        this.repository = repository;
+    public MavenMetadataLoader(CacheAwareExternalResourceAccessor cacheAwareExternalResourceAccessor, FileStore<String> resourcesFileStore) {
+        this.cacheAwareExternalResourceAccessor = cacheAwareExternalResourceAccessor;
+        this.resourcesFileStore = resourcesFileStore;
     }
 
-    public MavenMetadata load(URI metadataLocation) throws ResourceException {
+    public MavenMetadata load(ExternalResourceName metadataLocation) throws ResourceException {
         MavenMetadata metadata = new MavenMetadata();
         try {
             parseMavenMetadataInfo(metadataLocation, metadata);
         } catch (MissingResourceException e) {
             throw e;
         } catch (Exception e) {
-            throw new ResourceException(metadataLocation, String.format("Unable to load Maven meta-data from %s.", metadataLocation), e);
+            throw new ResourceException(metadataLocation.getUri(), String.format("Unable to load Maven meta-data from %s.", metadataLocation), e);
         }
         return metadata;
     }
 
-    private void parseMavenMetadataInfo(final URI metadataLocation, final MavenMetadata metadata) {
-        ExternalResource resource = repository.getResource(metadataLocation, true);
+    private void parseMavenMetadataInfo(final ExternalResourceName metadataLocation, final MavenMetadata metadata) throws IOException {
+        ExternalResource resource = cacheAwareExternalResourceAccessor.getResource(metadataLocation, new CacheAwareExternalResourceAccessor.ResourceFileStore() {
+            @Override
+            public LocallyAvailableResource moveIntoCache(File downloadedResource) {
+                String key = Hashing.sha1().hashString(metadataLocation.toString(), Charsets.UTF_8).toString();
+                return resourcesFileStore.move(key, downloadedResource);
+            }
+        }, null);
         if (resource == null) {
-            throw new MissingResourceException(metadataLocation, String.format("Maven meta-data not available: %s", metadataLocation));
-        }
-        try {
-            parseMavenMetadataInto(resource, metadata);
-        } finally {
-            resource.close();
+            throw new MissingResourceException(metadataLocation.getUri(), String.format("Maven meta-data not available at %s", metadataLocation));
         }
+        parseMavenMetadataInto(resource, metadata);
     }
 
     private void parseMavenMetadataInto(ExternalResource metadataResource, final MavenMetadata mavenMetadata) {
