@@ -24,7 +24,6 @@ import com.google.common.collect.Sets;
 import net.rubygrapefruit.platform.SystemInfo;
 import net.rubygrapefruit.platform.WindowsRegistry;
 import org.gradle.api.JavaVersion;
-import org.gradle.api.Nullable;
 import org.gradle.api.specs.Spec;
 import org.gradle.integtests.fixtures.executer.GradleDistribution;
 import org.gradle.integtests.fixtures.executer.UnderDevelopmentGradleDistribution;
@@ -39,6 +38,7 @@ import org.gradle.internal.os.OperatingSystem;
 import org.gradle.testfixtures.internal.NativeServicesTestFixture;
 import org.gradle.util.CollectionUtils;
 
+import javax.annotation.Nullable;
 import java.io.File;
 import java.util.ArrayList;
 import java.util.Arrays;
@@ -182,6 +182,20 @@ public abstract class AvailableJavaHomes {
         });
     }
 
+    /**
+     * Returns a JDK that has a different Java home to the current one, is supported byt the Gradle version under tests and a valid jre.
+     */
+    public static Jvm getDifferentJdkWithValidJre() {
+        return AvailableJavaHomes.getAvailableJdk(new Spec<JvmInstallation>() {
+            @Override
+            public boolean isSatisfiedBy(JvmInstallation jvm) {
+                return !jvm.getJavaHome().equals(Jvm.current().getJavaHome())
+                    && isSupportedVersion(jvm)
+                    && Jvm.discovered(jvm.getJavaHome(), jvm.getJavaVersion()).getJre() != null;
+            }
+        });
+    }
+
     /**
      * Locates a JRE installation for the current JVM. Prefers a stand-alone JRE installation over one that is part of a JDK install.
      *
