@@ -23,7 +23,6 @@ import java.util.List;
 
 public class DefaultJavaCompileSpec extends DefaultJvmLanguageCompileSpec implements JavaCompileSpec {
     private CompileOptions compileOptions;
-    private File dependencyCacheDir;
     private List<File> annotationProcessorPath;
 
     @Override
@@ -35,18 +34,6 @@ public class DefaultJavaCompileSpec extends DefaultJvmLanguageCompileSpec implem
         this.compileOptions = compileOptions;
     }
 
-    @Deprecated
-    @Override
-    public File getDependencyCacheDir() {
-        return dependencyCacheDir;
-    }
-
-    @Deprecated
-    @Override
-    public void setDependencyCacheDir(File dependencyCacheDir) {
-        this.dependencyCacheDir = dependencyCacheDir;
-    }
-
     @Override
     public List<File> getAnnotationProcessorPath() {
         return annotationProcessorPath;
