@@ -18,12 +18,11 @@ package org.gradle.nativeplatform.toolchain.internal.gcc;
 
 import org.gradle.api.Action;
 import org.gradle.api.GradleException;
-import org.gradle.api.internal.tasks.SimpleWorkResult;
 import org.gradle.api.tasks.WorkResult;
 import org.gradle.internal.operations.BuildOperationExecutor;
 import org.gradle.internal.operations.BuildOperationQueue;
-import org.gradle.language.base.internal.compile.Compiler;
 import org.gradle.nativeplatform.internal.StaticLibraryArchiverSpec;
+import org.gradle.nativeplatform.toolchain.internal.AbstractCompiler;
 import org.gradle.nativeplatform.toolchain.internal.ArgsTransformer;
 import org.gradle.nativeplatform.toolchain.internal.CommandLineToolContext;
 import org.gradle.nativeplatform.toolchain.internal.CommandLineToolInvocation;
@@ -36,35 +35,16 @@ import java.util.List;
 /**
  * A static library archiver based on the GNU 'ar' utility
  */
-class ArStaticLibraryArchiver implements Compiler<StaticLibraryArchiverSpec> {
-    private final CommandLineToolInvocationWorker commandLineToolInvocationWorker;
-    private final ArgsTransformer<StaticLibraryArchiverSpec> argsTransformer = new ArchiverSpecToArguments();
-    private final CommandLineToolContext invocationContext;
-    private final BuildOperationExecutor buildOperationExecutor;
-
+class ArStaticLibraryArchiver extends AbstractCompiler<StaticLibraryArchiverSpec> {
     ArStaticLibraryArchiver(BuildOperationExecutor buildOperationExecutor, CommandLineToolInvocationWorker commandLineToolInvocationWorker, CommandLineToolContext invocationContext) {
-        this.buildOperationExecutor = buildOperationExecutor;
-        this.commandLineToolInvocationWorker = commandLineToolInvocationWorker;
-        this.invocationContext = invocationContext;
+        super(buildOperationExecutor, commandLineToolInvocationWorker, invocationContext, new ArchiverSpecToArguments(), false);
     }
 
     @Override
     public WorkResult execute(final StaticLibraryArchiverSpec spec) {
         deletePreviousOutput(spec);
 
-        List<String> args = argsTransformer.transform(spec);
-        invocationContext.getArgAction().execute(args);
-        final CommandLineToolInvocation invocation = invocationContext.createInvocation(
-                "archiving " + spec.getOutputFile().getName(), args, spec.getOperationLogger());
-
-        buildOperationExecutor.runAll(commandLineToolInvocationWorker, new Action<BuildOperationQueue<CommandLineToolInvocation>>() {
-            @Override
-            public void execute(BuildOperationQueue<CommandLineToolInvocation> buildQueue) {
-                buildQueue.setLogLocation(spec.getOperationLogger().getLogLocation());
-                buildQueue.add(invocation);
-            }
-        });
-        return new SimpleWorkResult(true);
+        return super.execute(spec);
     }
 
     private void deletePreviousOutput(StaticLibraryArchiverSpec spec) {
@@ -77,6 +57,26 @@ class ArStaticLibraryArchiver implements Compiler<StaticLibraryArchiverSpec> {
         }
     }
 
+    @Override
+    protected Action<BuildOperationQueue<CommandLineToolInvocation>> newInvocationAction(final StaticLibraryArchiverSpec spec) {
+        List<String> args = getArguments(spec);
+        final CommandLineToolInvocation invocation = newInvocation(
+            "archiving " + spec.getOutputFile().getName(), spec.getOutputFile().getParentFile(), args, spec.getOperationLogger());
+
+        return new Action<BuildOperationQueue<CommandLineToolInvocation>>() {
+            @Override
+            public void execute(BuildOperationQueue<CommandLineToolInvocation> buildQueue) {
+                buildQueue.setLogLocation(spec.getOperationLogger().getLogLocation());
+                buildQueue.add(invocation);
+            }
+        };
+    }
+
+    @Override
+    protected void addOptionsFileArgs(List<String> args, File tempDir) {
+        // No support for command file
+    }
+
     private static class ArchiverSpecToArguments implements ArgsTransformer<StaticLibraryArchiverSpec> {
         @Override
         public List<String> transform(StaticLibraryArchiverSpec spec) {
